@@ -29,7 +29,7 @@ static SyncSource *createSource(const SyncSourceParams &params)
     bool isMe = sourceType.m_backend == "apple-contacts";
 
 #ifndef ENABLE_ADDRESSBOOK
-    return isMe ? RegisterSyncSource::InactiveSource : NULL;
+    return isMe ? RegisterSyncSource::InactiveSource(params) : NULL;
 #else
     bool maybeMe = sourceType.m_backend == "addressbook";
     
