@@ -13,6 +13,8 @@
 
 #include <syncevo/LogRedirect.h>
 
+#include <boost/assign.hpp>
+
 #include <stdio.h>
 #include <errno.h>
 
@@ -274,21 +276,17 @@ const std::string *WebDAVSource::setResourceName(const std::string &item, std::s
     }
 
     // first check if the item already contains the right UID
-    std::string uid = extractUID(item);
+    size_t start, end;
+    std::string uid = extractUID(item, &start, &end);
     if (uid == olduid) {
         return &item;
     }
 
     // insert or overwrite
     buffer = item;
-    size_t start = buffer.find(UID);
-    if (start != buffer.npos) {
-        start += UID.size();
-        size_t end = buffer.find("\n", start);
-        if (end != buffer.npos) {
-            // overwrite
-            buffer.replace(start, end, olduid);
-        }
+    if (start != std::string::npos) {
+        // overwrite
+        buffer.replace(start, end - start, olduid);
     } else {
         // insert
         start = buffer.find("\nEND:" + getContent());
@@ -302,19 +300,53 @@ const std::string *WebDAVSource::setResourceName(const std::string &item, std::s
 
 
 
-std::string WebDAVSource::extractUID(const std::string &item)
+std::string WebDAVSource::extractUID(const std::string &item, size_t *startp, size_t *endp)
 {
     std::string luid;
+    if (startp) {
+        *startp = std::string::npos;
+    }
+    if (endp) {
+        *endp = std::string::npos;
+    }
     // find UID, use that plus ".vcf" as resource name (expected by Yahoo Contacts)
     size_t start = item.find(UID);
     if (start != item.npos) {
         start += UID.size();
         size_t end = item.find("\n", start);
         if (end != item.npos) {
+            if (startp) {
+                *startp = start;
+            }
             luid = item.substr(start, end - start);
             if (boost::ends_with(luid, "\r")) {
                 luid.resize(luid.size() - 1);
             }
+            // keep checking for more lines because of folding
+            while (end + 1 < item.size() &&
+                   item[end + 1] == ' ') {
+                start = end + 1;
+                end = item.find("\n", start);
+                if (end == item.npos) {
+                    // incomplete, abort
+                    luid = "";
+                    if (startp) {
+                        *startp = std::string::npos;
+                    }
+                    break;
+                }
+                luid += item.substr(start, end - start);
+                if (boost::ends_with(luid, "\r")) {
+                    luid.resize(luid.size() - 1);
+                }
+            }
+            // success, return all information
+            if (endp) {
+                // don't include \r or \n
+                *endp = item[end - 1] == '\r' ?
+                    end - 1 :
+                    end;
+            }
         }
     }
     return luid;
@@ -1156,6 +1188,16 @@ void WebDAVSource::getSynthesisInfo(SynthesisInfo &info,
                                     XMLConfigFragments &fragments)
 {
     TrackingSyncSource::getSynthesisInfo(info, fragments);
+
+    // only CalDAV enforces unique UID
+    std::string content = getContent();
+    if (content == "VEVENT" || content == "VTODO" || content == "VJOURNAL") {
+        info.m_globalIDs = true;
+    }
+    if (content == "VEVENT") {
+        info.m_backendRule = "HAVE-SYNCEVOLUTION-EXDATE-DETACHED";
+    }
+
     // TODO: instead of identifying the peer based on the
     // session URI, use some information gathered about
     // it during open()
@@ -1227,6 +1269,8 @@ std::string WebDAVSource::databaseRevision()
         return "";
     }
 
+    contactServer();
+
     Timespec deadline = createDeadline();
     Neon::Session::PropfindPropCallback_t callback =
         boost::bind(&WebDAVSource::openPropCallback,
@@ -1298,7 +1342,7 @@ void WebDAVSource::listAllItems(RevisionMap_t &revisions)
             Neon::XMLParser parser;
             parser.initReportParser(boost::bind(&WebDAVSource::checkItem, this,
                                                 boost::ref(revisions),
-                                                _1, _2, boost::ref(data)));
+                                                _1, _2, &data));
             parser.pushHandler(boost::bind(Neon::XMLParser::accept, "urn:ietf:params:xml:ns:caldav", "calendar-data", _2, _3),
                                boost::bind(Neon::XMLParser::append, boost::ref(data), _2, _3));
             Neon::Request report(*getSession(), "REPORT", getCalendar().m_path, query, parser);
@@ -1311,6 +1355,83 @@ void WebDAVSource::listAllItems(RevisionMap_t &revisions)
     }
 }
 
+std::string WebDAVSource::findByUID(const std::string &uid,
+                                    const Timespec &deadline)
+{
+    RevisionMap_t revisions;
+    std::string query;
+    if (getContent() == "VCARD") {
+        // use CardDAV
+        query =
+            "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
+            "<C:addressbook-query xmlns:D=\"DAV:\"\n"
+            "xmlns:C=\"urn:ietf:params:xml:ns:carddav:addressbook\">\n"
+            "<D:prop>\n"
+            "<D:getetag/>\n"
+            "</D:prop>\n"
+            "<C:filter>\n"
+            "<C:comp-filter name=\"" + getContent() + "\">\n"
+            "<C:prop-filter name=\"UID\">\n"
+            "<C:text-match>" + uid + "</C:text-match>\n"
+            "</C:prop-filter>\n"
+            "</C:comp-filter>\n"
+            "</C:filter>\n"
+            "</C:addressbook-query>\n";
+    } else {
+        // use CalDAV
+        query =
+            "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
+            "<C:calendar-query xmlns:D=\"DAV:\"\n"
+            "xmlns:C=\"urn:ietf:params:xml:ns:caldav\">\n"
+            "<D:prop>\n"
+            "<D:getetag/>\n"
+            "</D:prop>\n"
+            "<C:filter>\n"
+            "<C:comp-filter name=\"VCALENDAR\">\n"
+            "<C:comp-filter name=\"" + getContent() + "\">\n"
+            "<C:prop-filter name=\"UID\">\n"
+            // Collation from RFC 4791, not supported yet by all servers.
+            // Apple Calendar Server did not like CDATA.
+            // TODO: escape special characters.
+            "<C:text-match" /* collation=\"i;octet\" */ ">" /* <[CDATA[ */ + uid + /* ]]> */ "</C:text-match>\n"
+            "</C:prop-filter>\n"
+            "</C:comp-filter>\n"
+            "</C:comp-filter>\n"
+            "</C:filter>\n"
+            "</C:calendar-query>\n";
+    }
+    getSession()->startOperation("REPORT 'UID lookup'", deadline);
+    while (true) {
+        Neon::XMLParser parser;
+        parser.initReportParser(boost::bind(&WebDAVSource::checkItem, this,
+                                            boost::ref(revisions),
+                                            _1, _2, (std::string *)0));
+        Neon::Request report(*getSession(), "REPORT", getCalendar().m_path, query, parser);
+        report.addHeader("Depth", "1");
+        report.addHeader("Content-Type", "application/xml; charset=\"utf-8\"");
+        if (report.run()) {
+            break;
+        }
+    }
+
+    switch (revisions.size()) {
+    case 0:
+        SE_THROW_EXCEPTION_STATUS(TransportStatusException,
+                                  "object not found",
+                                  SyncMLStatus(404));
+        break;
+    case 1:
+        return revisions.begin()->first;
+        break;
+    default:
+        // should not happen
+        SE_THROW(StringPrintf("UID %s not unique?!", uid.c_str()));
+    }
+
+    // not reached
+    return "";
+}
+
 void WebDAVSource::listAllItemsCallback(const Neon::URI &uri,
                                         const ne_prop_result_set *results,
                                         RevisionMap_t &revisions,
@@ -1353,7 +1474,7 @@ void WebDAVSource::listAllItemsCallback(const Neon::URI &uri,
 int WebDAVSource::checkItem(RevisionMap_t &revisions,
                             const std::string &href,
                             const std::string &etag,
-                            std::string &data)
+                            std::string *data)
 {
     // Ignore responses with no data: this is not perfect (should better
     // try to figure out why there is no data), but better than
@@ -1361,20 +1482,23 @@ int WebDAVSource::checkItem(RevisionMap_t &revisions,
     //
     // One situation is the response for the collection itself,
     // which comes with a 404 status and no data with Google Calendar.
-    if (data.empty()) {
+    if (data && data->empty()) {
         return 0;
     }
 
     // No need to parse, user content cannot start at start of line in
     // iCalendar 2.0.
-    if (data.find("\nBEGIN:" + getContent()) != data.npos) {
+    if (!data ||
+        data->find("\nBEGIN:" + getContent()) != data->npos) {
         std::string davLUID = path2luid(Neon::URI::parse(href).m_path);
         std::string rev = ETag2Rev(etag);
         revisions[davLUID] = rev;
     }
 
     // reset data for next item
-    data.clear();
+    if (data) {
+        data->clear();
+    }
     return 0;
 }
 
@@ -1469,7 +1593,8 @@ TrackingSyncSource::InsertItemResult WebDAVSource::insertItem(const string &uid,
             req.addHeader("If-None-Match", "*");
         }
         req.addHeader("Content-Type", contentType().c_str());
-        if (!req.run()) {
+        static const std::set<int> expected = boost::assign::list_of(412);
+        if (!req.run(&expected)) {
             goto retry;
         }
         SE_LOG_DEBUG(NULL, NULL, "add item status: %s",
@@ -1482,6 +1607,16 @@ TrackingSyncSource::InsertItemResult WebDAVSource::insertItem(const string &uid,
         case 201:
             // created
             break;
+        case 412: {
+            // "Precondition Failed": our only precondition is the one about
+            // If-None-Match, which means that there must be an existing item
+            // with the same UID. Go find it, so that we can report back the
+            // right luid.
+            std::string uid = extractUID(item);
+            std::string luid = findByUID(uid, deadline);
+            return InsertItemResult(luid, "", ITEM_NEEDS_MERGE);
+            break;
+        }
         default:
             SE_THROW_EXCEPTION_STATUS(TransportStatusException,
                                       std::string("unexpected status for insert: ") +
@@ -1665,19 +1800,9 @@ void WebDAVSource::removeItem(const string &uid)
         // TODO: match exactly the expected revision, aka ETag,
         // or implement locking.
         // req.addHeader("If-Match", etag);
-        try {
-            if (req->run()) {
-                break;
-            }
-        } catch (const TransportStatusException &ex) {
-            if (ex.syncMLStatus() == 412) {
-                // Radicale reports 412 'Precondition Failed'. Hmm, okay.
-                // Let's map it to the expected 404.
-                SE_THROW_EXCEPTION_STATUS(TransportStatusException,
-	                                  "object not found (was 412 'Precondition Failed')",
-	                                  SyncMLStatus(404));
-            }
-            throw;
+        static const std::set<int> expected = boost::assign::list_of(412);
+        if (req->run(&expected)) {
+            break;
         }
     }
     SE_LOG_DEBUG(NULL, NULL, "remove item status: %s",
@@ -1689,6 +1814,13 @@ void WebDAVSource::removeItem(const string &uid)
     case 200:
         // reported by Radicale, also okay
         break;
+    case 412:
+        // Radicale reports 412 'Precondition Failed'. Hmm, okay.
+        // Let's map it to the expected 404.
+        SE_THROW_EXCEPTION_STATUS(TransportStatusException,
+                                  "object not found (was 412 'Precondition Failed')",
+                                  SyncMLStatus(404));
+        break;
     default:
         SE_THROW_EXCEPTION_STATUS(TransportStatusException,
                                   std::string("unexpected status for removal: ") +
