@@ -1,13 +1,13 @@
 /**
  *  @File     localengineds.h
  *
- *  @Author   Lukas Zeller (luz@synthesis.ch)
+ *  @Author   Lukas Zeller (luz@plan44.ch)
  *
  *  @brief TLocalEngineDS
  *    Abstraction of the local datastore - interface class to the
  *    sync engine.
  *
- *    Copyright (c) 2001-2009 by Synthesis AG (www.synthesis.ch)
+ *    Copyright (c) 2001-2011 by Synthesis AG + plan44.ch
  *
  *  @Date 2005-09-15 : luz : created from localdatastore
  */
@@ -117,6 +117,8 @@ protected:
 extern const TFuncTable ClientDBFuncTable;
 #endif
 
+typedef list<string> TStringList;
+
 // local datastore config
 class TLocalDSConfig: public TConfigElement
 {
@@ -137,22 +139,30 @@ public:
   TConflictResolution fFirstTimeStrategy;
   // - options
   bool fReadOnly;          // if set, datastore will not write any data (only maps) to local DB (but fake successful status to remote)
+  bool fCanRestart;        // if set, then the datastore is able to participate in multiple sync sessions; in other words after a successful read/write cycle it is possible to restart at the reading phase
+  set<string> fSyncModes;  // all additional, non-standard sync modes
+#ifndef HARDCODED_CONFIG
+  string fSyncModeBuffer;  // only used during XML config parsing
+#endif
   bool fReportUpdates;     // if set(normal case), updates of server items will be sent to client (can be set to false for example for emails)
   bool fResendFailing;     // if set, items that receive a failure status from the remote will be resent in the next session (if DS 1.2 suspend marks supported by the DB)
   bool fDeleteWins;        // if set, in a replace/delete conflict the delete wins (also see DELETEWINS())
-  #ifndef SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   bool fTryUpdateDeleted;  // if set, in a client update with server delete conflict, server tries to update the already deleted item (in case it is just invisible)
   bool fAlwaysSendLocalID; // always send localID to clients (which gives them opportunity to remap IDs on Replace)
-  #endif
+  TStringList fAliasNames; // list of aliases for this datastore
+  #endif // SYSYNC_SERVER
   uInt32 fMaxItemsPerMessage; // if >0, limits the number of items sent per SyncML message (useful in case of slow datastores where collecting data might exceed client timeout)
   #ifdef OBJECT_FILTERING
   // filtering
   // - filter applied to items coming from remote party, non-matching
   //   items will be rejected with 415 (unknown format) status
+  //   unless fSilentlyDiscardUnaccepted is set to true (then we accept and discard silently)
   //   (outgoing items will be made pass this filter with makePassFilter)
   string fRemoteAcceptFilter;
+  bool fSilentlyDiscardUnaccepted;
   // - filter will be applied to items read from the local database,
-  //   IN ADDITION to eventually specified target address filtering
+  //   IN ADDITION to possibly specified target address filtering
   string fLocalDBFilterConf;
   // - filter applied to incoming items with makePassFilter
   //   when there is no fSyncSetFilter or when applying fSyncSetFilter
@@ -200,6 +210,10 @@ public:
   virtual void addTypeLimits(TLocalEngineDS *aLocalDatastoreP, TSyncSession *aSessionP);
   // - reset config to defaults
   virtual void clear();
+  // - check for alias names
+  uInt16 isDatastoreAlias(cAppCharP aDatastoreURI);
+  // - returns true for datastores that are abstract, i.e. don't have a backend implementation (like superdatastores, or non-derived localengineds)
+  virtual bool isAbstractDatastore(void) { return true; }; // pure localengineds is abstract. First derivate towards backend (stdlogicds) will override this with "false".
 protected:
   // check config elements
   #ifndef HARDCODED_CONFIG
@@ -233,7 +247,7 @@ typedef enum {
   dssta_syncmodestable,           ///< client&server, sync mode is now stable (all alerts and alert statuses are exchanged and processed), server is ready for early maps
   dssta_dataaccessstarted,        ///< client&server, user data access has started (e.g. loading of sync set in progress)
   dssta_syncsetready,             ///< client&server, sync set is ready and can be accessed with logicXXXX sync op calls
-  dssta_clientsyncgenstarted,     ///< client only, generation of sync command(s) in client for server has started (eventually, we send some pending maps first)
+  dssta_clientsyncgenstarted,     ///< client only, generation of sync command(s) in client for server has started (possibly, we send some pending maps first)
   dssta_serverseenclientmods,     ///< server only, server has seen all client modifications now
   dssta_serversyncgenstarted,     ///< server only, generation of sync command(s) in server for client has started
   dssta_syncgendone,              ///< client&server, generation of sync command(s) is complete
@@ -295,11 +309,7 @@ class TLocalEngineDS: public TSyncDataStore
   friend class TSyncOpCommand;
   friend class TMultiFieldItemType;
   friend class TTextProfileHandler;
-  #ifndef SYSYNC_CLIENT
-  friend class TSyncServer;
-  #else
-  friend class TSyncClient;
-  #endif
+  friend class TSyncAgent;
   #ifdef SUPERDATASTORES
   friend class TSuperDataStore;
   #endif
@@ -310,7 +320,7 @@ protected:
   /// @{
   TLocalEngineDSState fLocalDSState;  ///< internal state of the datastore sync process
   localstatus fAbortStatusCode;       ///< status code when engAbortDatastoreSync() was called
-  bool fLocalAbortCause;							///< flag signalling if abort cause was local or remote
+  bool fLocalAbortCause;              ///< flag signalling if abort cause was local or remote
   bool fRemoteAddingStopped;          ///< set when no more add commands should be sent to remote (e.g. device full)
   localstatus fAlertCode;             ///< alert code in use by this datastore (for scripts and for suspend)
   /// @}
@@ -349,9 +359,10 @@ protected:
   // - other state info
   bool fFirstTimeSync;          ///< set to true if this is the first sync
   // - item ID lists (maps)
-  #ifndef SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   TStringToStringMap fTempGUIDMap;      ///< container for temp GUID to real GUID mapping
-  #else
+  #endif
+  #ifdef SYSYNC_CLIENT
   TStringToStringMap fPendingAddMaps;   ///< container for map items to be sent to server: fPendingAddMaps[localid]=remoteid; Note: might contain temporary localIDs that must be converted to final ones before using in <map> or saving with dsFinalizeLocalID()
   TStringToStringMap fUnconfirmedMaps;  ///< container for map items already sent to server, but not yet confirmed: fUnconfirmedMaps[localid]=remoteid;
   TStringToStringMap fLastSessionMaps;  ///< container for map items already confirmed, but still needed for duplicate checking:: fLastSessionMaps[localid]=remoteid;
@@ -469,7 +480,7 @@ protected:
   sInt32 fRemoteItemsDeleted;
   sInt32 fRemoteItemsError; // items that had a remote error (and will be resent later)
   /// }@
-  #ifndef SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   // @{ conflicts
   sInt32 fConflictsServerWins;
   sInt32 fConflictsClientWins;
@@ -493,16 +504,13 @@ private:
   // - local
   string fLocalDBPath;              ///< client, entire path to the local database, which might include subpaths/CGI to subselect records in the local DB
   // - remote
-  string fRemoteDBPath;             ///< client, path of remote DB, including all CGI params etc.
   string fDBUser;                   ///< client, DB layer user name
   string fDBPassword;               ///< client, DB layer password
   string fRemoteRecordFilterQuery;  ///< client, record level filter query
   bool fRemoteFilterInclusive;      ///< client, inclusive filter flag
-  #else
-  #ifndef MINIMAL_CODE
-  string fRemoteDBPath;             ///< server, path of remote DB, for documentary purposes
-  #endif
-  #endif
+  #endif // SYSYNC_CLIENT
+  string fRemoteDBPath;             ///< server and client, path of remote DB, for documentary purposes
+  string fIdentifyingDBName;        ///< server and client, name with which the to-be-synced DB is identified (important in case of aliases in server)
   /// remote datastore involved, valid after processSyncCmdAsServer()
   TRemoteDataStore *fRemoteDatastoreP;
   /// Remote view if local URI
@@ -542,6 +550,8 @@ public:
   bool dbgTestState(TLocalEngineDSState aMinState, bool aNeedExactMatch=false)
     { return aNeedExactMatch ? fLocalDSState==aMinState : fLocalDSState>=aMinState; };
   #endif
+  // get name with which this datastore was identified by the remote
+  cAppCharP getIdentifyingName(void) { return fIdentifyingDBName.c_str(); }
   /// calculate Sync mode and flags from given alert code
   /// @note does not affect DS state in any way, nor checks if DS can handle this code.
   ///   (this will be done only when calling setSyncMode())
@@ -553,6 +563,10 @@ public:
   // called for >=SyncML 1.1 if remote wants number of changes.
   // Must return -1 if no NOC value can be returned
   virtual sInt32 getNumberOfChanges(void) { return -1; /* no NOC supported */ };
+  /// Called at end of sync to determine whether the store already knows
+  /// that it has more changes for the server in the next sync session.
+  /// For example, the TBinFileImplDS looks at its change log to determine that.
+  virtual bool hasPendingChangesForNextSync() { return false; }
   /// test abort status, datastore is aborted also when session is just suspended
   bool isAborted(void);
   /// abort status code with local error code prefix if cause was local
@@ -590,10 +604,8 @@ public:
   TSyncItemType *getLocalReceiveType(void) { return fLocalReceiveFromRemoteTypeP; };    ///< type used by local to receive from remote
   TSyncItemType *getRemoteSendType(void) { return fRemoteSendToLocalTypeP; };           ///< type used by remote to send to local
   TSyncItemType *getRemoteReceiveType(void) { return fRemoteReceiveFromLocalTypeP; };   ///< type used by remote to receive from local
-  #if defined(SYSYNC_CLIENT) || !defined(MINIMAL_CODE)
   /// get remote DB path
   cAppCharP getRemoteDBPath(void) { return fRemoteDBPath.c_str(); };
-  #endif
   #ifdef SYSYNC_CLIENT
   /// get local DB path
   cAppCharP getLocalDBPath(void) { return fLocalDBPath.c_str(); };
@@ -603,8 +615,8 @@ public:
   /// datastore options
   fieldinteger_t getItemSizeLimit(void) { return fItemSizeLimit; };
   bool getNoAttachments(void) {
-	  #ifdef SYSYNC_TARGET_OPTIONS
-  	return fNoAttachments;
+    #ifdef SYSYNC_TARGET_OPTIONS
+    return fNoAttachments;
     #else
     return false;
     #endif
@@ -664,15 +676,19 @@ public:
   #ifdef SYSYNC_CLIENT
   /// initialize Sync alert for datastore according to Parameters set with dsSetClientSyncParams()
   /// @note initializes anchors and makes calls to isFirstTimeSync() valid
-  localstatus engPrepareClientSyncAlert(TSuperDataStore *aAsSubDatastoreOf);
+  SUPERDS_VIRTUAL localstatus engPrepareClientSyncAlert(void);
+  /// internal helper to be called by engPrepareClientSyncAlert() from this class and from superdatastore
+  localstatus engPrepareClientDSForAlert(void);
   /// generate Sync alert for datastore
   /// @note this could be repeatedly called due to auth failures at beginning of session
   /// @note this is a NOP for subDatastores (should not be called in this case, anyway)
   localstatus engGenerateClientSyncAlert(TAlertCommand *&aAlertCommandP);
-  // Init engine for client sync
+  // Init engine for client sync (superdatastore aware)
   // - determine types to exchange
   // - make sync set ready
-  localstatus engInitForClientSync(void);
+  SUPERDS_VIRTUAL localstatus engInitForClientSync(void);
+  // - non-superdatastore aware base functionality
+  localstatus engInitDSForClientSync(void);
   #endif
   /// Internal events during sync for derived classes
   /// @note local DB authorisation must be established already before calling these
@@ -704,6 +720,8 @@ public:
     const char *aTargetURIOptions,      ///< option string contained in target URI
     bool aFromSyncCommand=false         ///< must be set when parsing options from <sync> target URI
   );
+  /// reset all filter settings
+  void resetFiltering(void);
   /// process SyncML 1.2 style filter
   localstatus engProcessDS12Filter(SmlFilterPtr_t aTargetFilter);
   /// initialize reception of syncop commands for datastore
@@ -727,20 +745,17 @@ public:
   /// @param[in] aNotYetInMapPackage set if we are still in sync-from-server package
   /// @note usually, client starts sending maps while still receiving syncops from server
   void engClientStartOfMapMessage(bool aNotYetInMapPackage);
-  /// called to generate Map items
-  /// @note Returns true if now finished for this datastore
-  /// @note also sets fState to dss_done when finished
-  SUPERDS_VIRTUAL bool generateMapItems(TMapCommand *aMapCommandP);
   /// Client only: returns number of unsent map items
   SUPERDS_VIRTUAL sInt32 numUnsentMaps(void);
-  #else
+  #endif // SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   /// server only: called whenever we should start a sync command for all alerted datastores
   /// if not already started
   void engServerStartOfSyncMessage(void);
   /// called to process map commands from client to server
   /// @note aLocalID or aRemoteID can be NULL - which signifies deletion of a map entry
   SUPERDS_VIRTUAL localstatus engProcessMap(cAppCharP aRemoteID, cAppCharP aLocalID);
-  #endif
+  #endif // SYSYNC_SERVER
   /// check is datastore is completely started.
   /// @param[in] aWait if set, call will not return until either started state is reached
   ///   or cannot be reached within the maximally allowed request processing time left.
@@ -782,7 +797,8 @@ public:
     TSyncItem *aSyncItemP, ///< the item to be processed
     TStatusCommand &aStatusCommand ///< status, must be set to correct status code (ok / error)
   );
-  #else
+  #endif
+  #ifdef SYSYNC_SERVER
   bool engProcessRemoteItemAsServer(
     TSyncItem *aSyncItemP, ///< the item to be processed
     TStatusCommand &aStatusCommand ///< status, must be set to correct status code (ok / error)
@@ -797,10 +813,10 @@ public:
   /// called to mark maps confirmed, that is, we have received ok status for them
   #ifdef SYSYNC_CLIENT
   SUPERDS_VIRTUAL void engMarkMapConfirmed(cAppCharP aLocalID, cAppCharP aRemoteID);
-  /// Client only: called to generate Map items
-  /// - Returns true if now finished for this datastore
-  /// - also sets fState to dss_done when finished
-  SUPERDS_VIRTUAL bool engGenerateMapItems(TMapCommand *aMapCommandP);
+  /// called to generate Map items
+  /// @note Returns true if now finished for this datastore
+  /// @note also sets fState to done when finished
+  SUPERDS_VIRTUAL bool engGenerateMapItems(TMapCommand *aMapCommandP, cAppCharP aLocalIDPrefix);
   /// Client only: Check if the remoteid was used by an add command not
   /// fully mapped&confirmed in the previous session
   bool isAddFromLastSession(cAppCharP aRemoteID);
@@ -840,7 +856,7 @@ public:
   const char *filterCGIToString(cAppCharP aCGI, string &aFilter);
   */
   /// @brief parse "syncml:filtertype-cgi" filter, convert into internal filter syntax
-  ///  and eventually sets some special filter options (fDateRangeStart, fDateRangeEnd)
+  ///  and possibly sets some special filter options (fDateRangeStart, fDateRangeEnd)
   ///  based on "filterkeywords" available for the type passed (DS 1.2).
   ///  For parsing DS 1.1/1.0 TAF-style filters, aItemType can be NULL, no type-specific
   ///  filterkeywords can be parsed then.
@@ -879,7 +895,7 @@ public:
   #endif
   // get usage variant for a specified type usage
   virtual TTypeVariantDescriptor getVariantDescForType(TSyncItemType *aItemTypeP);
-  #ifndef SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   /// called at end of request processing in server (derived only by superdatastore)
   SUPERDS_VIRTUAL void engRequestEnded(void);
   /// end map operation (derived class might want to rollback)
@@ -911,7 +927,7 @@ public:
   virtual TSyError TunnelReadItemAsKey(cItemID aID, KeyH aItemKey) { return LOCERR_NOTIMP; };
   virtual TSyError TunnelInsertItemAsKey(KeyH aItemKey, ItemID aID) { return LOCERR_NOTIMP; };
   virtual TSyError TunnelUpdateItemAsKey(KeyH aItemKey, cItemID aID, ItemID aUpdID) { return LOCERR_NOTIMP; };
-  
+
   virtual TSettingsKeyImpl *newTunnelKey(TEngineInterface *) { return NULL; };
 
   /// @}
@@ -938,7 +954,7 @@ public:
     const char *aRecordFilterQuery = NULL,
     bool aFilterInclusive = false
   );
-  #endif
+  #endif // SYSYNC_CLIENT
 protected:
   /// reset datastore to a re-usable, like new-created state.
   virtual void dsResetDataStore(void) {};
@@ -972,7 +988,7 @@ protected:
   /// returns true if DB implementation supports resuming in midst of a chunked item (can save fPIxxx.. and related admin data)
   virtual bool dsResumeChunkedSupportedInDB(void) { return false; };
   /// called when a item in the sync set changes its localID (due to local DB internals)
-  /// Datastore must make sure that eventually cached items get updated
+  /// Datastore must make sure that possibly cached items get updated
   virtual void dsLocalIdHasChanged(const char *aOldID, const char *aNewID);
   /// called when request processing ends
   virtual void dsEndOfMessage(void) {}; // nop at this level
@@ -1001,9 +1017,10 @@ protected:
 
   /// get conflict resolution strategy.
   virtual TConflictResolution getConflictStrategy(bool aForSlowSync, bool aForFirstTime=false);
-  #ifndef SYSYNC_CLIENT
-  /// called to check if conflicting replace command from server exists
+  #ifdef SYSYNC_SERVER
+  /// check if conflicting item already exist in list of items-to-be-sent-to-client
   virtual TSyncItem *getConflictingItemByRemoteID(TSyncItem *syncitemP) = 0;
+  virtual TSyncItem *getConflictingItemByLocalID(TSyncItem *syncitemP) = 0;
   /// called to check if content-matching item from server exists
   virtual TSyncItem *getMatchingItem(TSyncItem *syncitemP, TEqualityMode aEqMode) = 0;
   /// called to prevent item to be sent to client in subsequent engGenerateSyncCommands()
@@ -1038,7 +1055,8 @@ protected:
     TSmlCommand * &aInterruptedCommandP,
     const char *aLocalIDPrefix=NULL
   ) = 0;
-  #else
+  #endif
+  #ifdef SYSYNC_SERVER
   /// server: called to generate sync sub-commands to be sent to client
   /// Returns true if now finished for this datastore
   /// also sets fState to dss_syncdone(server)/dss_syncready(client) when finished
@@ -1058,7 +1076,7 @@ protected:
   /// called to mark an already sent item as "to-be-resent", e.g. due to temporary
   /// error status conditions, by localID or remoteID (latter only in server case).
   virtual void logicMarkItemForResend(cAppCharP aLocalID, cAppCharP aRemoteID) = 0;
-  /// save status information required to eventually perform a resume (as passed to datastore with
+  /// save status information required to possibly perform a resume (as passed to datastore with
   /// markOnlyUngeneratedForResume() and markItemForResume())
   /// (or, in case the session is really complete, make sure that no resume state is left)
   /// @note Must also save tempGUIDs (for server) and pending/unconfirmed maps (for client)
@@ -1104,14 +1122,15 @@ private:
   TRuleMatchTypesContainer fRuleMatchItemTypes; // contains rule match item types
   #endif
 protected:
-  #ifndef SYSYNC_CLIENT
+  #ifdef SYSYNC_SERVER
   /// for sending GUIDs (Add command), generate temp GUID which conforms to maxguidsize of remote datastore if needed
   void adjustLocalIDforSize(string &aLocalID, sInt32 maxguidsize, sInt32 prefixsize);
   /// for received GUIDs (Map command), obtain real GUID (might be temp GUID due to maxguidsize restrictions)
   void obtainRealLocalID(string &aLocalID);
-  /// helper to force a conflict (i.e. have a particular item in the sync set)
-  TSyncItem *forceConflict(TSyncItem *aSyncItemP);
-  #endif
+  /// helper to cause database version of an item (as identified by aSyncItemP's ID) to be sent to client
+  /// (aka "force a conflict")
+  TSyncItem *SendDBVersionOfItemAsServer(TSyncItem *aSyncItemP);
+  #endif // SYSYNC_SERVER
   /// helper to save resume state either at end of request or explicitly at reception of a "suspend"
   SUPERDS_VIRTUAL localstatus engSaveSuspendState(bool aAnyway);
   /// Returns true if type information is sufficient to create items to be sent to remote party
@@ -1123,7 +1142,8 @@ protected:
   /// helper for derived classes to generate sync op commands
   TSyncOpCommand *newSyncOpCommand(
     TSyncItem *aSyncItemP, // the sync item
-    TSyncItemType *aSyncItemTypeP // the sync item type
+    TSyncItemType *aSyncItemTypeP, // the sync item type
+    cAppCharP aLocalIDPrefix // prefix for localID (can be NULL for none)
   );
   /// return pure relative (item) URI (removes absolute part or ./ prefix)
   /// @note this one is virtual because it is defined in TSyncDataStore
@@ -1141,6 +1161,13 @@ protected:
     string *aTableNameP=NULL,
     string *aCGIP=NULL
   );
+  /// get DB specific error code for last routine call that returned !=LOCERR_OK
+  /// @return platform specific DB error code
+  virtual uInt32 lastDBError(void) { return 0; };
+  virtual bool isDBError(uInt32 aErrCode) { return aErrCode!=0; } // standard implementation assumes 0=ok
+  /// get error message text showing lastDBError for dbg log, or empty string if none
+  /// @return platform specific DB error text
+  virtual string lastDBErrorText(void);
   /// @}
 }; // TLocalEngineDS
 
